package com.iona.fuse.demo.logisticx.service.order.support;

/**
 * 
 * Copyright  2007-2008 IONA Technologies PLC.
 *           All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 **/

import java.util.List;

import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.xml.namespace.QName;
import org.apache.servicemix.jbi.jaxp.StringSource;

import com.iona.fuse.demo.logisticx.model.LineItem;
import com.iona.fuse.demo.logisticx.model.Order;
import com.iona.fuse.demo.logisticx.model.OrderStatus;

/** 
 * A helper class for the order web service. This is the worker bee for the 
 * {@link com.iona.fuse.demo.logisticx.service.order.OrderService} web service.
 * 
 * @author bsnyder
 * @author enolan
 *
 */
public class OrderServiceSupport {

    /** The namespace to use for calling any other services **/
    private static final String LOGISTICX_NAMESPACE = "http://logisticx.demo.fuse.iona.com/";

    /** The service name of the processor service **/
    private static final String PROCESSOR_SERVICE_NAME = "processorService";
    
    /** The name of the order processor endpoint on the processor service **/
    private static final String ORDER_PROCESSOR_ENDPOINT_NAME = "orderProcessor";
    
    /** The service name of the persistence service **/
    private static final String PERSISTENCE_SERVICE_NAME = "persistenceService";
    
    /** The name of the order persistence endpoint on the persistence service **/
    private static final String PERSISTENCE_ENDPOINT_NAME = "orderPersister"; 

    /** A JPA query to retreive the order status based on the order id. **/
    private static final String ORDERSTATUS_QUERY = 
        "select stat from com.iona.fuse.demo.logisticx.model.OrderStatus stat where stat.order.orderId = ?1";
            
    /** The JBI <a href="http://servicemix.apache.org/dist/servicemix-3.2.1/site/parent/core/servicemix-jbi/apidocs/javax/jbi/component/ComponentContext.html">
     * component context</a> to provide access to items such as the {@link DeliveryChannel}, {@link ServiceEndpoint}s, 
     * the endpoint registry and other important JBI 
     */ 
    private ComponentContext context;

    public OrderServiceSupport() {
    }
    
    public ComponentContext getContext() {
        return context;
    }

    public void setContext(ComponentContext context) {
        this.context = context;
    }

    public void persist(Order order) {
        doJpaPersist(order);
    }
    
    public void persist(OrderStatus status) {
        doJpaPersist(status);
    }
 
    public OrderStatus retrieve(long orderId) {
        return doJpaRetrieve(orderId);
    }        

    public void send(OrderStatus status) {               
        doSend(status.getOrder().getOrderId(), status.getOrder());
    }

    /**
     * Send the orderId to the order processing service for further processing. 
     * 
     * @param orderId
     */
    private void doSend(long orderId, Order order) { 
        QName processorServiceName = new QName(LOGISTICX_NAMESPACE, PROCESSOR_SERVICE_NAME);
        
        // Resolve the target endpoint using the JBI component context  
        ServiceEndpoint se = 
            getContext().getEndpoint(processorServiceName, ORDER_PROCESSOR_ENDPOINT_NAME);

        OrderStatus stat = doJpaRetrieve(orderId); 
       
        try {
            // Create an in-only message exchange because a response is not necessary
            InOnly exchange =
                getContext().getDeliveryChannel().createExchangeFactory().createInOnlyExchange();
            exchange.setEndpoint(se);
            
            // Create a normalized message to send to the order processor 
            NormalizedMessage msg = exchange.createMessage();
            msg.setContent(getOrderIdContent(orderId));
            QName opName = new QName("http://logisticx.demo.fuse.iona.com/orderService/", "putOrder");
            exchange.setOperation(opName);
            exchange.setMessage(msg, "in");
            exchange.setService(processorServiceName);
            
            // Send the message exchange in an asynchronous manner 
            getContext().getDeliveryChannel().send(exchange);
        } catch (MessagingException e) {
            e.printStackTrace();
        }
    }

    /** 
     * Wrap the order id in XML to be used as the content of a normalized message. 
     * 
     * @param orderId
     * @return
     */
    private StringSource getOrderIdContent(long orderId) {
        StringBuffer message = new StringBuffer("<order-id>");
        message.append(orderId);
        message.append("</order-id>");
        System.out.printf("Using orderId [%s]", orderId);

        return new StringSource(message.toString());
    }        
    
    /** 
     * Persists the order in the database for additional downstream processing. To be 
     * replaced with <a href="http://activemq.apache.org/camel/jpa.html">Camel JPA</a>
     *  
     * @param object The order to persist
     */
    private void doJpaPersist(Object object){
        EntityManagerFactory emFactory = Persistence.createEntityManagerFactory("order-processor");        
        EntityManager em = emFactory.createEntityManager();
        try {
            em.getTransaction().begin();
            
            if (object instanceof Order) {
                em.persist((Order) object);
            } else {
                em.persist((OrderStatus) object);
            }
            em.getTransaction().commit();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
            emFactory.close();
        }
    }
    
    /**
     * Query for an order based on the given <tt>orderId</tt> and return 
     * an OrderStatus object. To be replaced with <a href="http://activemq.apache.org/camel/jpa.html">
     * Camel JPA</a>.
     * 
     * @param orderId
     * @return The status of the order 
     */
    private OrderStatus doJpaRetrieve(Long orderId) {       
        OrderStatus status = null;
        EntityManagerFactory emFactory = Persistence.createEntityManagerFactory("order-processor");
        EntityManager em = emFactory.createEntityManager();
        try {
            em.getTransaction().begin();                       
            Query orderQuery = em.createQuery(ORDERSTATUS_QUERY);
            orderQuery.setParameter(1, orderId);
            status = (OrderStatus) orderQuery.getSingleResult();
            em.getTransaction().commit();
        } catch (Exception e) { 
            e.printStackTrace();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
            emFactory.close();
        }
        
        return status;
    }
    
}
